* Cmp: concentration of solute in the matrix in equilibrium with the 
* precipitate
* Cpm: concentration of solute in the precipitate in equilibrium with the 
* matrix
* Cbulk(i): bulk concentration of the ith slice
* Cbulkf(i): bulk concentration of the ith slice for the previous step
* Cmpr0: concentration of solute in the matrix in equilibrium with the
* precipitate for the a precipitate radius of rad0
* Cmpr(i): concentration of solute in the matrix in equilibrium with the
* precipitate for the a precipitate radius of rad(i)
* Cmpinf: concentration of solute in the matrix in equilibrium with the
* precipitate for the an infinite precipitate radius(Grain boundary)  
* rad0: initial mean radius of the precipitates 
* rad(i): mean radius of the precipitates of the ith slice
* dt: time step 
* dx: slice thickness 
* D: diffusion coefficient 
* N: number density of precipitate 
* sizeG: size of the grain 
* Vmol: molar volume of the precipitate 
* Temp: temperature 
* Smp: matrix/precipitate interface energy per unit area
* Invelp: interface velocity planar, at the interface between the grain 
* boundary and the first slice
* Invels: interface velocity spherical, used for the dissolution of the 
* precipitates
* deltV: global volume of the precipitates before dissolution-global
* volume of the precipitates after dissolution 
* deltx: increase of the thickness of the grain boundary precipitate 
* V: volume of the slices   
* deltVp: volume increase of the grain boundary for each time step 
* t: time
* maxtim: maximum time  



      program coarseVG
      

      implicit none
      
      integer dim

      parameter (dim=100)

      double precision Pi
                  
      double precision Cbulk, Cbulkf, rad, Cmpr, super
      dimension Cbulk(dim) 
      dimension Cbulkf(dim)
      dimension rad(dim)
      dimension Cmpr(dim)
      dimension super(dim)
             
      double precision Cpm, Cmpinf, Xpm, Xmpinf, Xbulk, V
      double precision D, N, Vmol, Temp, Smp, sizeG, dt, dx
      double precision Invelp, Invels, deltVp, deltV

      double precision rad0, Cmpr0, Cb1, Cb2, Cb3, Cb4
      double precision rad1, radl, radn 
      double precision t, deltx
      double precision maxtim
      
      integer nbstpz, nbstpt, p
      integer i, j, k, l, m, ii, jj, ll 
      
      parameter (Temp=923.D0, D=1.8D-11)
      parameter (Smp=0.2D0, Vmol=5.84D-6, rad0=0.75D-6, sizeG=50.D-6)
      parameter (Xpm=0.25D0, Xmpinf=3.9D-4, Xbulk=0.005D0) 
      parameter (dt=1.D-3, dx=1.5D-6, maxtim=36.D4)               
      
      Pi=dacos(-1.D0)

      nbstpz=Int((sizeG/(2.D0))/dx)
      nbstpt=Int(maxtim/dt)

      V=dx
      call Gibbs_Thomson (Xmpinf, Xpm, Smp, Vmol, rad0, Temp, 
     & Cmpr0)
      N=(Xbulk-(Cmpr0*Vmol))/Xpm
      N=N/((4.D0/3.D0)*Pi*rad0*rad0*rad0)
      deltx=0.D0
      p=0
      Cpm=Xpm/Vmol
      Cmpinf=Xmpinf/Vmol

* initialization of the radius and concentration values 

      do 10 i=1, nbstpz
         rad(i)=rad0
         Cbulk(i)=Cmpr0
         Cbulkf(i)=Cmpr0
         Cmpr(i)=Cmpr0
         super(i)=0.D0
 10   continue
      
     

* program
      
      open (16, file='Cbulkm')
      open (17, file='Radm')
      open (18, file='supersatm')

      do 20 k=0, nbstpt 
         p=p+1
         t=(dble(k))*dt
         super(1)=(Cbulkf(1)-Cmpinf)/(Cpm-Cmpinf)

         if (k .eq. 0) then
           write (16,110) t, deltx, (Cbulk(ii),ii=1,7), Cbulk(nbstpz)
           write (17,110) t, deltx, (rad(jj),jj=1,7), rad(nbstpz)
           write (18,110) t, (super(ll), ll=1,nbstpz)
           goto 20
         endif 


         Invelp=(D/V)*super(1)
        
         deltVp=Invelp*dt
         deltx=deltx+deltVp

         call finite_difference_central (Cbulkf(1), Cbulkf(2), Cmpinf
     &    , D, dt, dx, Cbulk(1))

         if (rad(1) .eq. 0.D0) then
            goto 40
         endif
                  
         call Interface_velocity_spherical (Cbulk(1), D, rad(1),
     &    Cpm, Cmpr(1), Invels)

         call decrease_vol (N, V, rad(1), Invels, dt, deltV)
         
         call decrease_rad_precip (rad(1), Invels, dt, N, V, deltV,
     &    rad1)
         rad(1)=rad1

         call new_compo(deltV, V, Cpm, Cmpr(1), Cbulk(1), Cb2)
         Cbulk(1)=Cb2

         if (rad(1) .eq. 0.D0) then
            goto 35
         endif   
        
         call Gibbs_Thomson (Xmpinf, Xpm, Smp, Vmol, rad(1),
     &    Temp, Cmpr(1))

                             
 35      if (k .eq. 1) then
            goto 100
         endif
         
         
        
 40      do 30 l=2, nbstpz-1

            call finite_difference_central (Cbulkf(l), Cbulkf(l+1),
     &       Cbulkf(l-1), D, dt, dx, Cbulk(l))
                                   
            if (rad(l) .eq. 0.D0) then
               goto 30
            endif
            
            super(l)=(Cbulk(l)-Cmpr(l))/(Cpm-Cmpr(l))
            
            call Interface_velocity_spherical (Cbulk(l), D, rad(l)
     &       , Cpm, Cmpr(l), Invels)

            call decrease_vol (N, V, rad(l), Invels, dt, deltV)

            call decrease_rad_precip (rad(l), Invels, dt, N, V, deltV,
     &       radl)
            rad(l)=radl

            call new_compo (deltV, V, Cpm, Cmpr(l),Cbulk(l), Cb3)
            Cbulk(l)=Cb3

            if (rad(l) .eq. 0.D0) then
               goto 30
            endif   

            call Gibbs_Thomson (Xmpinf, Xpm, Smp, Vmol, rad(l)
     &       , Temp, Cmpr(l))
             
                   
 30      continue

         call finite_difference_reflec (Cbulkf(nbstpz),Cbulkf(nbstpz-1),
     &     D, dt, dx, Cbulk(nbstpz))
                                    
         if (rad(nbstpz) .eq. 0.D0) then
            goto 100
         endif
         
         super(nbstpz)=(Cbulk(nbstpz)-Cmpr(nbstpz))/(Cpm-Cmpr(nbstpz))

           
         call Interface_velocity_spherical (Cbulk(nbstpz), D,rad(nbstpz)
     &    , Cpm, Cmpr(nbstpz), Invels)

         call decrease_vol (N, V, rad(nbstpz), Invels, dt, deltV)

         call decrease_rad_precip (rad(nbstpz), Invels, dt, N, V, deltV,
     &    radn)
         rad(nbstpz)=radn

         call new_compo (deltV, V, Cpm, Cmpr(nbstpz), Cbulk(nbstpz),Cb4)
         Cbulk(nbstpz)=Cb4

         if (rad(nbstpz) .eq. 0.D0) then
            goto 100
         endif   

         call Gibbs_Thomson (Xmpinf, Xpm, Smp, Vmol, rad(nbstpz)
     &    , Temp, Cmpr(nbstpz))

                      
         
 100     if (p .eq. 100001) then
            write (16,110) t, deltx, (Cbulk(ii),ii=1,7), Cbulk(nbstpz)
            write (17,110) t, deltx, (rad(jj),jj=1,7), rad(nbstpz)
            write (18,110) t, (super(ll), ll=1,nbstpz)
            p=1
         endif
   
*         if (t .eq. (1.D0*3600.D0*1.D4))  then
*            do 120 ii=1,nbstpz
*               write (18,140) ii, rad(ii), Cbulk(ii)
* 120        continue
*         endif

         do 50 j=1, nbstpz
            Cbulkf(j)=Cbulk(j)
  50      continue
   
 110     format (f15.2,2X,15(d20.10,2X))
 140     format (f15.2,2X,3(d20.10,2X))

 20   continue

 600  close(16)
      close(17)
      close(18)
      end


*                              ********

* Subroutine Gibbs_Thomson gives the mole fraction of solute in the matrix in 
* equilibrium with the precipitate, function of the radius of the precipitate.  


      subroutine Gibbs_Thomson (Xmps, Xpms, Smps, Vmols, rads, 
     & Temps, Cmprs)
      
      double precision Xmps, Xpms, Smps, Vmols, rads, Temps
      double precision Cmprs
      double precision R

      R=8.31451D0

      Cmprs=(2.D0*Vmols*Smps)/(R*Temps*(Xpms-Xmps))
      Cmprs=dexp(Cmprs/rads)
      Cmprs=Xmps*Cmprs
      Cmprs=Cmprs/Vmols

      end


*                              ********

  
* Subroutine Interface_velocity_spherical 

      
      subroutine Interface_velocity_spherical (Cbulks, Ds, rads, Cpms,
     & Cmprs, Intvs)
     
      double precision Cbulks, Ds, rads, Cpms, Cmprs
      double precision Intvs
      double precision supsat
      
      supsat=(Cbulks-Cmprs)/(Cpms-Cmprs)
      Intvs=(supsat*Ds)
      Intvs=Intvs/(rads)
      
      end


*                              ********

* Subroutine decrease_vol calculates the volume loss of the precipitates in V
 


      subroutine decrease_vol (Ns, Vs, rads, Intvs, dts, deltVs)
     
      double precision Ns, Vs, rads, Intvs, dts
      double precision Pi, deltVs

      Pi=dacos(-1.D0)
      
      deltVs=Ns*Vs*(4.D0)*Pi*rads*rads*Intvs*dts

      end


*                              ********

* Subroutine decrease_rad_precip gives the new precipitate radius after
* dissolution (step by step)


      subroutine decrease_rad_precip (rbdiss, Intvs, dts, Ns, Vs,
     & deltVs, radiss)
     
      double precision rbdiss, Intvs, dts, Ns, Vs, Pi
      double precision deltVs, radiss

      Pi=dacos(-1.D0)

      radiss=rbdiss*rbdiss*rbdiss
      radiss=radiss+((3.D0)*rbdiss*rbdiss*Intvs*dts)

      if (radiss .le. 0.D0) then
         radiss=0.D0
         deltVs=Ns*Vs*(4.D0/3.D0)*Pi*(rbdiss*rbdiss*rbdiss)
         else
         radiss=radiss**((1.D0)/(3.D0))
      endif
   
      end


*                              ********

* Subroutine new_compo, assuming that the molar volumes are equal.


      subroutine new_compo (deltVs, Vs, Ca, Cb, Cform, Cnew)
      
      implicit none
                     
      double precision deltVs, Vs, Cform, Ca, Cb
      double precision Cnew
      double precision deltCs

      deltCs=-(deltVs/(Vs))*(Ca-Cb)
      Cnew=Cform+deltCs

      end




*                              ********

* Subroutine finite_difference_central


      Subroutine finite_difference_central (Cx1t1, Cx2t1, Cx0t1,
     & Ds, dts, dxs, Cx1t2)
     
      double precision Cx1t1, Cx2t1, Cx0t1, Ds, dts, dxs
      double precision Cx1t2
 
      Cx1t2=Ds*dts*(Cx2t1-(2.D0*Cx1t1)+Cx0t1)
      Cx1t2=Cx1t2/(dxs*dxs)
      Cx1t2=Cx1t1+Cx1t2

      end



*                              ********

* Subroutine finite_difference_reflec


      Subroutine finite_difference_reflec (Cx1t1, Cx0t1,
     & Ds, dts, dxs, Cx1t2)
     
      double precision Cx1t1, Cx0t1, Ds, dts, dxs
      double precision Cx1t2
 
      Cx1t2=2.D0*Ds*dts*(Cx0t1-Cx1t1)
      Cx1t2=Cx1t2/(dxs*dxs)
      Cx1t2=Cx1t1+Cx1t2

      end
